package com.boman.system.service;

import java.util.List;
import java.util.Set;


import com.boman.common.core.web.domain.AjaxResult;
import com.boman.system.api.domain.SysMenu;
import com.boman.system.domain.vo.RouterVo;
import com.boman.system.domain.vo.TreeSelect;

/**
 * 菜单 业务层
 * 
 * @author ruoyi
 */
public interface ISysMenuService
{
    /**
     * 根据用户查询系统菜单列表
     * 
     * @param userId 用户ID
     * @return 菜单列表
     */
    public List<SysMenu> selectMenuList(Long userId);

    /**
     * 根据用户查询系统菜单列表
     * 
     * @param menu 菜单信息
     * @param userId 用户ID
     * @return 菜单列表
     */
    public List<SysMenu> selectMenuList(SysMenu menu, Long userId);

    public List<SysMenu> selectMenuListAll(SysMenu menu);

    /**
     * 根据用户ID查询权限
     * 
     * @param id 用户ID
     * @return 权限列表
     */
    public Set<String> selectMenuPermsById(Long id);

    /**
     * 根据用户ID查询菜单树信息
     * 
     * @param id 用户ID
     * @return 菜单列表
     */
    public List<SysMenu> selectMenuTreeById(Long id);

    /**
     * 根据用户ID查询菜单树信息
     *
     * @param menuId menuId
     * @return 菜单列表
     */
    List<SysMenu> allLeafNodeById(Long menuId);

    /**
     * 根据角色ID查询菜单树信息
     * 
     * @param roleId 角色ID
     * @return 选中菜单列表
     */
    public List<Integer> selectMenuListByRoleId(Long roleId);

    /**
     * 构建前端路由所需要的菜单
     * 
     * @param menus 菜单列表
     * @return 路由列表
     */
    public List<RouterVo> buildMenus(List<SysMenu> menus);

    /**
     * 构建前端所需要树结构
     * 
     * @param menus 菜单列表
     * @return 树结构列表
     */
    public List<SysMenu> buildMenuTree(List<SysMenu> menus);
    /**
     * 构建前端所需要树结构 没有叶子结点
     *
     * @param menus 菜单列表
     * @return 树结构列表
     */
    public List<SysMenu> buildMenuTreeNotAddLeafNode(List<SysMenu> menus);

    /**
     * 构建前端所需要下拉树结构
     * 
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildMenuTreeSelect(List<SysMenu> menus);

    /**
     * 构建前端所需要下拉树结构
     *
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildMenuTreeSelectNotAddLeafNode(List<SysMenu> menus);

    /**
     * 根据菜单ID查询信息
     * 
     * @param id 菜单ID
     * @return 菜单信息
     */
    public SysMenu selectMenuById(Long id);

    /**
     * 是否存在菜单子节点
     * 
     * @param id 菜单ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean hasChildById(Long id);

    /**
     * 查询菜单是否存在角色
     * 
     * @param id 菜单ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean checkMenuExistRole(Long id);

    /**
     * 新增保存菜单信息
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public AjaxResult insertMenu(SysMenu menu);

    /**
     * 修改保存菜单信息
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public int updateMenu(SysMenu menu);

    /**
     * 删除菜单管理信息
     * 
     * @param id 菜单ID
     * @return 结果
     */
    public int deleteMenuById(Long id);

    public int removeMenuById(Long id);

    /**
     * 校验菜单名称是否唯一
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public String checkMenuNameUnique(SysMenu menu);

}
